require 'spec_helper'

describe Date do
  let(:time_zone) { 'America/New_York' }

  shared_examples_for 'utc_offset' do
    it 'should get correct beginning of SG day for a timezone' do
      should == expected_offset
    end
  end

  describe '#utc_time_offset' do
    subject { time.utc_time_offset(time_zone) }

    context 'when US timezone' do
      let(:time_zone) { 'America/New_York' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { 9 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { 10 }
      end
    end

    context 'when IT timezone' do
      let(:time_zone) { 'Europe/Rome' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { 3 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { 4 }
      end
    end

    context 'when AU timezone' do
      let(:time_zone) { 'Australia/Sydney' }

      context 'when daylight time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-01-10 00:00 UTC') }
        let(:expected_offset) { -6 }
      end

      context 'when winter time' do
        it_should_behave_like 'utc_offset'
        let(:time) { Date.parse('2014-06-10 00:00 UTC') }
        let(:expected_offset) { -5 }
      end
    end
  end

  describe '#beginning_of_sg_day' do
    it 'should return sg offset for UTC' do
      expect(Date.parse('2013-07-08').beginning_of_sg_day('UTC')).to eq(Time.parse('2013-07-08 5:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(Date.parse('2014-03-08').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-08 10:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(Date.parse('2014-03-09').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(Date.parse('2014-11-01').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-01 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(Date.parse('2014-11-02').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end
  end

  describe '#end_of_sg_day' do
    it 'should return sg offset for UTC' do
      expect(Date.parse('2013-07-08').end_of_sg_day('UTC')).to eq(Time.parse('2013-07-09 5:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(Date.parse('2014-03-08').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(Date.parse('2014-03-09').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-10 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(Date.parse('2014-11-01').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(Date.parse('2014-11-02').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-03 10:00 UTC'))
    end
  end
end
