require 'spec_helper'

describe DateTime do
  let(:time_zone) { 'America/New_York' }

  describe '#beginning_of_sg_day' do
    it 'should return sg offset for UTC' do
      expect(DateTime.parse('2013-07-08').beginning_of_sg_day('UTC')).to eq(Time.parse('2013-07-08 5:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(DateTime.parse('2014-03-08').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-08 10:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(DateTime.parse('2014-03-09').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for daylight saving time' do
      expect(DateTime.parse('2014-11-01').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-01 09:00 UTC'))
    end

    it 'should return beginning_of_sg_day for standard time (ET)' do
      expect(DateTime.parse('2014-11-02').beginning_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end
  end

  describe '#end_of_sg_day' do
    it 'should return sg offset for UTC' do
      expect(DateTime.parse('2013-07-08').end_of_sg_day('UTC')).to eq(Time.parse('2013-07-09 5:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(DateTime.parse('2014-03-08').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-09 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(DateTime.parse('2014-03-09').end_of_sg_day(time_zone)).to eq(Time.parse('2014-03-10 09:00 UTC'))
    end

    it 'should return end_of_sg_day for daylight saving time' do
      expect(DateTime.parse('2014-11-01').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-02 10:00 UTC'))
    end

    it 'should return end_of_sg_day for standard time (ET)' do
      expect(DateTime.parse('2014-11-02').end_of_sg_day(time_zone)).to eq(Time.parse('2014-11-03 10:00 UTC'))
    end
  end
end
